# Generated by Django 2.1.5 on 2019-01-14 11:05

import django.contrib.auth.validators
from django.db import migrations, models
import django.utils.timezone
import manifest.accounts.managers
import manifest.accounts.models
import timezone_field.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('auth', '0009_alter_user_last_name_max_length'),
    ]

    operations = [
        migrations.CreateModel(
            name='User',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('password', models.CharField(max_length=128, verbose_name='password')),
                ('last_login', models.DateTimeField(blank=True, null=True, verbose_name='last login')),
                ('is_superuser', models.BooleanField(default=False, help_text='Designates that this user has all permissions without explicitly assigning them.', verbose_name='superuser status')),
                ('username', models.CharField(error_messages={'unique': 'A user with that username already exists.'}, help_text='Required. 150 characters or fewer. Letters, digits and @/./+/-/_ only.', max_length=150, unique=True, validators=[django.contrib.auth.validators.UnicodeUsernameValidator()], verbose_name='username')),
                ('first_name', models.CharField(blank=True, max_length=30, verbose_name='first name')),
                ('last_name', models.CharField(blank=True, max_length=150, verbose_name='last name')),
                ('email', models.EmailField(blank=True, max_length=254, verbose_name='email address')),
                ('is_staff', models.BooleanField(default=False, help_text='Designates whether the user can log into this admin site.', verbose_name='staff status')),
                ('is_active', models.BooleanField(default=True, help_text='Designates whether this user should be treated as active. Unselect this instead of deleting accounts.', verbose_name='active')),
                ('date_joined', models.DateTimeField(default=django.utils.timezone.now, verbose_name='date joined')),
                ('activation_key', models.CharField(blank=True, max_length=40, verbose_name='Activation key')),
                ('email_unconfirmed', models.EmailField(blank=True, help_text='Temporary email address when the user requests an email change.', max_length=254, verbose_name='Unconfirmed email address')),
                ('email_confirmation_key', models.CharField(blank=True, max_length=40, verbose_name='Unconfirmed email verification key')),
                ('email_confirmation_key_created', models.DateTimeField(blank=True, null=True, verbose_name='Creation date of email confirmation key')),
                ('gender', models.CharField(blank=True, choices=[('F', 'Female'), ('M', 'Male')], max_length=1, null=True, verbose_name='Gender')),
                ('birth_date', models.DateField(blank=True, null=True, verbose_name='Birth date')),
                ('picture', models.ImageField(blank=True, upload_to=manifest.accounts.models.upload_to_picture, verbose_name='Picture')),
                ('timezone', timezone_field.fields.TimeZoneField(verbose_name='Timezone')),
                ('locale', models.CharField(choices=[('en', 'English'), ('tr', 'Turkish')], default='en-us', max_length=10, verbose_name='Locale')),
                ('groups', models.ManyToManyField(blank=True, help_text='The groups this user belongs to. A user will get all permissions granted to each of their groups.', related_name='user_set', related_query_name='user', to='auth.Group', verbose_name='groups')),
                ('user_permissions', models.ManyToManyField(blank=True, help_text='Specific permissions for this user.', related_name='user_set', related_query_name='user', to='auth.Permission', verbose_name='user permissions')),
            ],
            options={
                'swappable': 'AUTH_USER_MODEL',
            },
            managers=[
                ('objects', manifest.accounts.managers.UserManager()),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
import random
import hashlib
from datetime import datetime
from io import StringIO  
from django import forms
from django.utils.translation import ugettext_lazy as _
from django.contrib.auth import authenticate, get_user_model
from django.contrib.auth.forms import UserCreationForm, AuthenticationForm
from django.db.models import Q

from django.forms.widgets import SelectDateWidget

from manifest.accounts import defaults

attrs_dict = {'class': 'required'}


class RegistrationForm(UserCreationForm):
    """
    Form for creating a new user account.

    Validates that the requested username and email is not already in use.
    Also requires the password to be entered twice.

    """
    username = forms.RegexField(label=_(u"Username"), 
                                regex=r'^\w+$', max_length=30,
                                widget=forms.TextInput(attrs=attrs_dict),                                
                                error_messages={'invalid': _(u'Username must '
                                    'contain only letters, numbers and underscores.')})

    email = forms.EmailField(label=_(u"Email address"), 
                                widget=forms.TextInput(
                                    attrs=dict(attrs_dict, maxlength=75)))

    password1 = forms.CharField(label=_("Password"), 
                                    widget=forms.PasswordInput(
                                        attrs=attrs_dict))
    password2 = forms.CharField(label=_("Password confirmation"), 
                                    widget=forms.PasswordInput(
                                        attrs=attrs_dict))
            
    class Meta:
        model = get_user_model()
        fields = ['username', 'email', 'password1', 'password2']

    def clean_username(self):
        """
        Validate that the username is unique and not listed 
        in ``defaults.ACCOUNTS_FORBIDDEN_USERNAMES`` list.
        
        """
        try: 
            user = get_user_model().objects.get(username=self.cleaned_data["username"])
        except get_user_model().DoesNotExist: 
            pass
        else: 
            raise forms.ValidationError(
                            self.error_messages['duplicate_username'])

        if self.cleaned_data['username'].lower() \
            in defaults.ACCOUNTS_FORBIDDEN_USERNAMES:
            raise forms.ValidationError(_(u'This username is not allowed.'))
        return self.cleaned_data['username']

    def clean_email(self):
        """ 
        Validate that the email address is unique. 
        
        """
        if get_user_model().objects.filter(
            Q(email__iexact=self.cleaned_data['email']) |
            Q(email_unconfirmed__iexact=self.cleaned_data['email'])):
            raise forms.ValidationError(_(u'This email address is already '
                        'in use. Please supply a different email.'))
        return self.cleaned_data['email']

    def save(self):
        """ 
        Creates a new user and account. Returns the newly created user. 
        
        """
        username, email, password = (self.cleaned_data['username'],
                                     self.cleaned_data['email'],
                                     self.cleaned_data['password1'])

        user = get_user_model().objects.create_user(username, email, password,
                        not defaults.ACCOUNTS_ACTIVATION_REQUIRED, 
                            defaults.ACCOUNTS_ACTIVATION_REQUIRED)
        return user

class RegistrationFormOnlyEmail(RegistrationForm):
    """
    Form for creating a new user account but not needing a username.

    This form is an adaptation of :class:`RegistrationForm`. It's used 
    when ``ACCOUNTS_WITHOUT_USERNAME`` setting is set to ``True``. 
    And thus the user is not asked to supply an username, but one is 
    generated for them. The user can than keep sign in by using their email.

    """
    def __init__(self, *args, **kwargs):
        super(RegistrationFormOnlyEmail, self).__init__(*args, **kwargs)
        del self.fields['username']

    def save(self):
        """ 
        Generate a random username before falling back to parent 
        register form. 
        
        """
        while True:
            username = hashlib.sha1(str(random.random()).encode('utf-8')).hexdigest()[:5]
            try:
                get_user_model().objects.get(username__iexact=username)
            except get_user_model().DoesNotExist: break

        self.cleaned_data['username'] = username
        return super(RegistrationFormOnlyEmail, self).save()

class RegistrationFormToS(RegistrationForm):
    """ 
    Add a Terms of Service button to the ``RegistrationForm``. 
    
    """
    tos = forms.BooleanField(widget=forms.CheckboxInput(attrs=attrs_dict),
                             label=_(u'I have read and agree to the '
                                'Terms of Service.'),
                             error_messages={'required': _(u'You must '
                                'agree to the terms to register.')})

def identification_field_factory(label, error_required):
    """
    A simple identification field factory which enable you to set the label.

    :param label:
        String containing the label for this field.

    :param error_required:
        String containing the error message if the field is left empty.

    """
    return forms.CharField(label=_(u"%(label)s") % {'label': label},
                           widget=forms.TextInput(attrs=attrs_dict),
                           max_length=75,
                           error_messages={'required': _(u"%(error)s") % 
                                            {'error': error_required}})

class AuthenticationForm(forms.Form):
    """
    A custom form where the identification can be a email address or username.

    """
    identification = identification_field_factory(_(u"Email"),
                                                  _(u"Either supply us with "
                                                    "your email or username."))
    password = forms.CharField(label=_(u"Password"),
                               widget=forms.PasswordInput(
                                    attrs=attrs_dict, render_value=False))
    remember_me = forms.BooleanField(widget=forms.CheckboxInput(),
                                     required=False,
                                     label=_(u'Remember me for %(days)s') % 
                        {'days': _(defaults.ACCOUNTS_REMEMBER_ME_DAYS[0])})

    def __init__(self, request=None, *args, **kwargs):
        """ 
        A custom init because we need to change the label if no 
        usernames is used 
        
        """
        #self.request = request
        self.user_cache = None
        super(AuthenticationForm, self).__init__(*args, **kwargs)
        if defaults.ACCOUNTS_WITHOUT_USERNAMES:
            self.fields['identification'] = identification_field_factory(
                                                _(u"Email address"),
                                                _(u"Please supply your "
                                                    "email address."))

    def clean(self):
        """
        Checks for the identification and password.

        If the combination can't be found will raise an invalid sign in error.

        """
        identification = self.cleaned_data.get('identification')
        password = self.cleaned_data.get('password')

        if identification and password:
            self.user_cache = authenticate(identification=identification, 
                                password=password)
            if self.user_cache is None:
                raise forms.ValidationError(_(u"Please enter a correct "
                        "username or email address and password. "
                        "Note that both fields are case-sensitive."))
        return self.cleaned_data

    def get_user(self):
        return self.user_cache
            
class EmailForm(forms.Form):
    email = forms.EmailField(label=_(u"New email"), required=True,
                                widget=forms.TextInput(
                                    attrs=dict(attrs_dict, maxlength=75)))

    def __init__(self, user, *args, **kwargs):
        """
        The current ``user`` is needed for initialisation of this form so
        that we can check if the email address is still free and not always
        returning ``True`` for this query because it's the users own email
        address.

        """
        super(EmailForm, self).__init__(*args, **kwargs)
        if not isinstance(user, get_user_model()):
            raise TypeError(_(u"User must be an instance of User"))
        else: self.user = user

    def clean_email(self):
        """ 
        Validate that the email is not already registered with another user.
        
        """
        if self.cleaned_data['email'].lower() == self.user.email:
            raise forms.ValidationError(_(u"You're already known under "
                                                "this email address."))
        if get_user_model().objects.filter(
                email__iexact=self.cleaned_data['email']).exclude(
                    email__iexact=self.user.email):
            raise forms.ValidationError(_(u'This email address is already '
                        'in use. Please supply a different email address.'))
        return self.cleaned_data['email']

    def save(self):
        """
        Save method calls :func:`user.change_email()` method which sends out 
        an email with an verification key to verify and with it enable this 
        new email address.

        """
        return self.user.change_email(self.cleaned_data['email'])

class NameForm(forms.ModelForm):
    first_name = forms.CharField(max_length=30, required=False, 
                        widget=forms.TextInput(attrs={'class':'text'}))
    last_name = forms.CharField(max_length=30, required=False, 
                        widget=forms.TextInput(attrs={'class':'text'}))
    class Meta:
        model = get_user_model()
        fields = ('first_name', 'last_name')

class ProfileForm(forms.ModelForm):
    """ Base form used for fields that are always required """

    GENDER_CHOICES = (
        ('F', _(u'Female')),
        ('M', _(u'Male')),
    )

    first_name = forms.CharField(label=_(u'First name'),
                                 max_length=30,
                                 required=False)
    last_name = forms.CharField(label=_(u'Last name'),
                                max_length=30,
                                required=False)

    gender = forms.ChoiceField(required=False, choices=GENDER_CHOICES, 
                    widget=forms.RadioSelect())
    birth_date = forms.DateField(required=False, 
                    widget=SelectDateWidget(
                        years=range(datetime.today().year-99, 
                                    datetime.today().year-10)))

    class Meta:
        model = get_user_model()
        exclude = ['user']

    def clean_picture(self):
        """
        Validates format and file size of uploaded profile picture.
        
        """
        if self.cleaned_data.get('picture'):
            picture_data = self.cleaned_data['picture']
            if 'error' in picture_data:
                raise forms.ValidationError(_(u'Upload a valid image. '
                            'The file you uploaded was either not an image '
                            'or a corrupted image.'))
                
            content_type = picture_data.content_type
            if content_type:
                main, sub = content_type.split('/')
                if not (main == 'image' 
                            and sub in defaults.ACCOUNTS_PICTURE_FORMATS):
                    raise forms.ValidationError(_(u'%s only.' % 
                                    defaults.ACCOUNTS_PICTURE_FORMATS))
        
            if picture_data.size > int(defaults.ACCOUNTS_PICTURE_MAX_FILE):
                raise forms.ValidationError(_(u'Image size is too big.'))
            return self.cleaned_data['picture']




# -*- coding: utf-8 -*-
from django.contrib.auth import get_user_model
from django.utils.translation import ugettext_lazy as _

from manifest.accounts import forms
from manifest.accounts import defaults
from manifest.accounts.tests.base import AccountsTestCase

class RegistrationFormTests(AccountsTestCase):
    """ Test the registration form. """
    fixtures = ['test']

    def test_registration_form(self):
        """
        Test that the ``RegistrationForm`` checks for unique usernames 
        and unique e-mail addresses.

        """
        invalid_data_dicts = [
            # Non-alphanumeric username.
            {'data': {'username': 'foo@bar',
                      'email': 'foo@example.com',
                      'password': 'foo',
                      'password2': 'foo',
                      'tos': 'on'},
             'error': ('username', [_(u'Username must contain only letters, numbers and underscores.')])},
             
            # Password is not the same
            {'data': {'username': 'foo2bar',
                      'email': 'bar@example.com',
                      'password1': 'foo',
                      'password2': 'foo2',
                      'tos': 'on'},
             'error': ('password2', [_("The two password fields didn't match.")])},

            # Already taken username
            #{'data': {'username': 'john',
            #          'email': 'johndoe@example.com',
            #          'password1': 'foo',
            #          'password2': 'foo',
            #          'tos': 'on'},
            # 'error': ('username', [_(u'A user with that username already exists.')])},
            
            # Already taken email
            {'data': {'username': 'johndoe',
                      'email': 'john@example.com',
                      'password': 'foo',
                      'password2': 'foo',
                      'tos': 'on'},
             'error': ('email', [_(u'This email address is already in use. Please supply a different email.')])},

            # Forbidden username
            {'data': {'username': 'register',
                      'email': 'register@example.com',
                      'password': 'foo',
                      'password2': 'foo2',
                      'tos': 'on'},
             'error': ('username', [_(u'This username is not allowed.')])},
        ]

        for invalid_dict in invalid_data_dicts:
            form = forms.RegistrationForm(data=invalid_dict['data'])
            self.assertFalse(form.is_valid())
            self.assertEqual(form.errors[invalid_dict['error'][0]],
                             invalid_dict['error'][1])


        # And finally, a valid form.
        form = forms.RegistrationForm(data={'username': 'foobar',
                                      'email': 'foobar@example.com',
                                      'password1': 'foo',
                                      'password2': 'foo',
                                      'tos': 'on'})

        self.assertTrue(form.is_valid())

class AuthenticationFormTests(AccountsTestCase):
    """ Test the ``AuthenticationForm`` """

    fixtures = ['test']

    def test_authentication_form(self):
        """
        Check that the ``AuthenticationForm`` requires both identification and password

        """
        invalid_data_dicts = [
            {'data': {'identification': '',
                      'password': 'inhalefish'},
             'error': ('identification', [_(u"Either supply us with your email or username.")])},
            {'data': {'identification': 'john',
                      'password': 'inhalefish'},
             'error': ('__all__', [_(u"Please enter a correct username or email address and password. Note that both fields are case-sensitive.")])}
        ]

        for invalid_dict in invalid_data_dicts:
            form = forms.AuthenticationForm(data=invalid_dict['data'])
            self.failIf(form.is_valid())
            self.assertEqual(form.errors[invalid_dict['error'][0]],
                             invalid_dict['error'][1])

        valid_data_dicts = [
            {'identification': 'john',
             'password': 'pass'},
            {'identification': 'john@example.com',
             'password': 'pass'}
        ]

        for valid_dict in valid_data_dicts:
            form = forms.AuthenticationForm(data=valid_dict)
            self.assertTrue(form.is_valid())

    def test_authentication_form_email(self):
        """
        Test that the signin form has a different label is
        ``ACCOUNTS_WITHOUT_USERNAME`` is set to ``True``

        """
        defaults.ACCOUNTS_WITHOUT_USERNAMES = True

        form = forms.AuthenticationForm(data={'identification': "john",
                                              'password': "test"})

        correct_label = _(u"Email address")
        self.assertEqual(form.fields['identification'].label,
                         correct_label)

        # Restore default settings
        defaults.ACCOUNTS_WITHOUT_USERNAMES = False

class RegistrationFormOnlyEmailTests(AccountsTestCase):
    """
    Test the :class:`RegistrationFormOnlyEmail`.

    This is the same form as :class:`RegistrationForm` but doesn't require an
    username for a successfull signup.

    """
    fixtures = ['test']

    def test_registration_form_only_email(self):
        """
        Test that the form has no username field. And that the username is
        generated in the save method

        """
        valid_data = {'email': 'johndoe@example.com',
                      'password1': 'pass',
                      'password2': 'pass'}

        form = forms.RegistrationFormOnlyEmail(data=valid_data)

        # Should have no username field
        self.failIf(form.fields.get('username', False))

        # Form should be valid.
        self.failUnless(form.is_valid())

        # Creates an unique username
        user = form.save()

        self.failUnless(len(user.username), 5)

class EmailFormTests(AccountsTestCase):
    """ Test the ``EmailForm`` """
    fixtures = ['test']

    def test_email_form(self):
        user = get_user_model().objects.get(pk=1)
        invalid_data_dicts = [
            # No change in e-mail address
            {'data': {'email': 'john@example.com'},
             'error': ('email', [_(u"You're already known under this email address.")])},
            # An e-mail address used by another
            {'data': {'email': 'jane@example.com'},
             'error': ('email', [_(u'This email address is already in use. Please supply a different email address.')])},
        ]
        for invalid_dict in invalid_data_dicts:
            form = forms.EmailForm(user, data=invalid_dict['data'])
            self.failIf(form.is_valid())
            self.assertEqual(form.errors[invalid_dict['error'][0]],
                             invalid_dict['error'][1])

        # Test a valid post
        form = forms.EmailForm(user,
                                     data={'email': 'john.doe@example.com'})
        self.failUnless(form.is_valid())

    def test_form_init(self):
        """ The form must be initialized with a ``User`` instance. """
        self.assertRaises(TypeError, forms.EmailForm, None)



# -*- coding: utf-8 -*-
from django.contrib import admin
from django.contrib.auth import get_user_model
from django.contrib.auth.admin import UserAdmin as BaseUserAdmin
from django.utils.translation import ugettext as _

from manifest.accounts import defaults


class UserAdmin(BaseUserAdmin):
    pass

#admin.site.unregister(get_user_model())
admin.site.register(get_user_model(), UserAdmin)



# -*- coding: utf-8 -*-
from django.conf import settings
from django.urls import reverse
from django.shortcuts import redirect
from django.contrib.auth import (authenticate, login as auth_login, logout, 
                                    get_user_model, REDIRECT_FIELD_NAME)
from django.contrib.auth.forms import PasswordChangeForm
from django.utils.decorators import method_decorator
from django.contrib.auth.views import LoginView, LogoutView
from django.contrib.auth.decorators import login_required
from django.contrib import messages
from django.utils.translation import ugettext as _
from django.http import Http404, HttpResponsePermanentRedirect
from django.views.generic import (View, ListView, DetailView, FormView, 
                                    CreateView, UpdateView, TemplateView)

from manifest.accounts.forms import (RegistrationForm, 
                                        RegistrationFormOnlyEmail, 
                                        AuthenticationForm,
                                        EmailForm, ProfileForm)
from manifest.accounts.decorators import secure_required
from manifest.accounts.utils import login_redirect
from manifest.accounts import signals as accounts_signals
from manifest.accounts import defaults


class ExtraContextMixin(View):
    """
    A mixin that passes ``extra_context`` dictionary as template context.
    
    """
    
    extra_context = {}

    def get_context_data(self, **kwargs):
        context = super(ExtraContextMixin, self).get_context_data(**kwargs)
        context.update(self.extra_context)
        return context
    
    
class SecureRequiredMixin(View):
    """
    A mixin that switches URL from http to https if ``ACCOUNTS_USE_HTTPS`` 
    setting is ``True``.
    """
    
    #@method_decorator(secure_required)
    def dispatch(self, request, *args, **kwargs):    
        if not request.is_secure():
            if getattr(settings, 'ACCOUNTS_USE_HTTPS', defaults.ACCOUNTS_USE_HTTPS):
                request_url = request.build_absolute_uri(request.get_full_path())
                secure_url = request_url.replace('http://', 'https://')
                return HttpResponsePermanentRedirect(secure_url)
        return super(SecureRequiredMixin, self).dispatch(request, *args, **kwargs)
    


class LoginRequiredMixin(View):
    """
    A mixin that redirects user to login form if not authenticated yet.
    
    """
    
    @method_decorator(login_required)
    def dispatch(self, request, *args, **kwargs):
        return super(LoginRequiredMixin, self).dispatch(request, 
                                                    *args, **kwargs)


class Register(CreateView, ExtraContextMixin, SecureRequiredMixin):
    """
    Register user with a username, email and password. 
    
    Users receives an email with an activation link to activate their 
    account if ``ACCOUNTS_ACTIVATION_REQUIRED`` setting is ``True``. 
    
    Redirects to ``success_url`` if it is given, else redirects to 
    ``accounts_register_complete`` view.
    
    """
    
    model = get_user_model()
    template_name = 'accounts/register.html'
    success_message = _(u'You have been registered.')
    
    def get_form_class(self):
        if self.form_class: return self.form_class
        elif defaults.ACCOUNTS_WITHOUT_USERNAMES:
            return RegistrationFormOnlyEmail
        else: return RegistrationForm
    
    def dispatch(self, request, *args, **kwargs):
        if request.user.is_authenticated: 
            return redirect(reverse('accounts_settings'))
        return super(Register, self).dispatch(request, *args, **kwargs)
    
    def form_valid(self, form):
        user = form.save()
        accounts_signals.registration_complete.send(sender=None, 
            user=user, request=self.request)
        if defaults.ACCOUNTS_USE_MESSAGES:
            messages.success(self.request, self.success_message, 
                fail_silently=True)
        if self.success_url: return redirect(self.success_url)
        else: return redirect(reverse('accounts_register_complete', 
                                kwargs={'username': user.username}))

#class Login(FormView, ExtraContextMixin, SecureRequiredMixin):
class Login(LoginView, SecureRequiredMixin):
    """
    Authenticate user by email or username with password. 
    
    When the identification is correct and the user ``is_active`` 
    user will be redirected to ``success_url`` if it is defined. 
    
    If ``success_url`` is not defined, the ``login_redirect`` function  
    will be called with the arguments ``REDIRECT_FIELD_NAME`` and an 
    instance of the ``User`` whois is trying the login. The returned 
    value of the function will be the URL that will be redirected to.

    Users can also select to be remembered for ``ACCOUNTS_REMEMBER_DAYS``.
    
    """
    
    form_class = AuthenticationForm
    template_name = 'accounts/login.html'
    success_message = _(u'You have been logged in.')
    
    def form_valid(self, form):
        user = form.get_user()
        if user.is_active:
            auth_login(self.request, user)
            if form.cleaned_data['remember_me']:
                self.request.session.set_expiry(
                    defaults.ACCOUNTS_REMEMBER_ME_DAYS[1] * 86400)
            else: self.request.session.set_expiry(0)
            if defaults.ACCOUNTS_USE_MESSAGES:
                messages.success(self.request, self.success_message, 
                    fail_silently=True)
            if self.success_url: 
                return redirect(self.success_url)
            else: 
                url = login_redirect(
                        self.request.GET.get(REDIRECT_FIELD_NAME), user)
                return redirect(url)
        else: 
            return redirect(reverse('accounts_disabled', 
                        kwargs={'username': user.username}))
                        

class Logout(TemplateView, ExtraContextMixin):
    """
    """
    
    template_name = 'accounts/logout.html'
    success_message = _(u'You have been logged out.')
    
    def get(self, request, *args, **kwargs):
        logout(request)
        return super(Logout, self).get(request, *args, **kwargs)            

class Activate(TemplateView, ExtraContextMixin):
    """
    Activate the user with the activation key.

    The key is a SHA1 string. When the SHA1 is found with username
    the ``User`` of that account will be activated.  
    
    After a successfull activation user will be redirected to
    ``accounts_profile_detail`` view if ``succes_url`` is not defined. 
    
    If the SHA1 is not found, the user will be shown the
    ``template_name`` template displaying a fail message.
        
    """
    
    template_name = 'accounts/activate_fail.html'
    success_message = _(u'Your account has been activated.')
    success_url = None

    def get_success_url(self, **kwargs):
        if self.success_url: return self.success_url % kwargs
        else: return reverse('accounts_profile_detail', 
                        kwargs={'username': self.kwargs['username']})
        
    def get(self, request, username, activation_key, *args, **kwargs):
        user = get_user_model().objects.activate_user(username, activation_key)
        if user:
            # Sign the user in.
            auth_login(request, user, backend='manifest.accounts.backends.AuthenticationBackend')
            if defaults.ACCOUNTS_USE_MESSAGES:
                messages.success(self.request, self.success_message, 
                                    fail_silently=True)
            return redirect(self.get_success_url(**kwargs))
        return super(Activate, self).get(request, *args, **kwargs)            

class ProfileUpdate(UpdateView, SecureRequiredMixin, LoginRequiredMixin):
    """
    Update profile of current user
    
    Updates profile information for ``request.user``. User will be
    redirected to ``accounts_settings`` view in ``success_url`` is not
    defined.

    """
    
    model = get_user_model()
    profile_form = ProfileForm
    fields = ['first_name', 'last_name', 'gender', 'birth_date', 'picture', 
                    'timezone', 'locale']
    template_name = 'accounts/profile_form.html'
    success_message = _(u'Your profile has been updated.')
    
    def get_object(self):
        return self.request.user

    def get_initial(self):
        return {'first_name': self.request.user.first_name, 
                'last_name': self.request.user.last_name}

    def form_valid(self, form):
        profile = form.save()
        if defaults.ACCOUNTS_USE_MESSAGES:
            messages.success(self.request, self.success_message, 
                fail_silently=True)
        if self.success_url: return redirect(self.success_url)
        else: return redirect(reverse('accounts_settings'))


class PasswordChange(FormView, SecureRequiredMixin, LoginRequiredMixin):
    """
    Change password of current user
    
    Changes password for ``request.user``. User will be redirected to
    ``accounts_password_change_done`` view if ``success_url`` is not defined.

    """
    
    form_class = PasswordChangeForm
    template_name = 'accounts/password_change_form.html'
    
    def get_form_kwargs(self, **kwargs):
        kwargs = super(PasswordChange, self).get_form_kwargs(**kwargs)
        kwargs['user'] = self.request.user
        return kwargs
        
    def form_valid(self, form):
        user = form.save()
        accounts_signals.password_complete.send(sender=None, user=user)
        if self.success_url: return redirect(self.success_url)
        else: return redirect(reverse('accounts_password_change_done', 
                                kwargs={'username': user.username}))
                    

class EmailChange(PasswordChange):
    """
    Change email of current user
    
    Changes email for ``request.user``. Change will not be applied 
    until user confirm their new email.
    
    User will be redirected to ``accounts_email_change_done`` view 
    if ``success_url`` is not defined.
            
    """

    form_class = EmailForm
    template_name = 'accounts/email_change_form.html'
    
    def form_valid(self, form):
        user = form.save()
        if self.success_url: return redirect(self.success_url)
        else: 
            return redirect(reverse('accounts_email_change_done', 
                                kwargs={'username': user.username}))


class EmailConfirm(Activate, ExtraContextMixin):
    """
    Confirm the email address with username and confirmation key.

    Confirms the new email address by running 
    ``get_user_model().objects.confirm_email`` method.
    
    User will be redirected to ``accounts_email_change_complete`` view 
    if ``success_url`` is not defined. 
    
    If no ``User`` object returned the user will be shown the
    ``template_name`` template displaying a fail message.
    
    """
    
    template_name = 'accounts/email_change_fail.html'
    
    def get_success_url(self, **kwargs):
        if self.success_url: return self.success_url % kwargs
        else: return reverse('accounts_email_change_complete', 
                        kwargs={'username': self.kwargs['username']})

    def get(self, request, username, confirmation_key, *args, **kwargs):
        user = get_user_model().objects.confirm_email(username,
                                                         confirmation_key)
        if user:
            return redirect(self.get_success_url(**kwargs))
        return super(EmailConfirm, self).get(request, username, 
                                                        confirmation_key, 
                                                        *args, **kwargs)


class UserTemplateView(DetailView, LoginRequiredMixin, ExtraContextMixin):
    """
    Template view for current user
    
    Simple detail view gets ``request.user`` object.
    
    """
    
    template_name='accounts/settings.html'
    
    def get_object(self):
        return self.request.user


class UserView(DetailView, ExtraContextMixin):

    """
    Template view for current user account
    
    Simple detail view gets account by ``username`` as object.

    """

    queryset = get_user_model().objects.select_related().all()
    template_name = "accounts/settings.html"
    slug_field = 'username'
    slug_url_kwarg = 'username'


class ProfileList(ListView, ExtraContextMixin):
    """
    List active user profiles
    
    Lists active user profiles if ``ACCOUNTS_DISABLE_PROFILE_LIST``
    setting is True. Otherwise raises Http404.
    
    """

    queryset = get_user_model().objects.get_visible_profiles()
    template_name = "accounts/profile_list.html"
    
    def dispatch(self, request, *args, **kwargs):
        if defaults.ACCOUNTS_DISABLE_PROFILE_LIST \
           and not request.user.is_superuser:
            raise Http404
        return super(ProfileList, self).dispatch(request, *args, **kwargs)
        

class ProfileDetail(UserView, ExtraContextMixin):
    """
    Shows active user profile
    
    Simple detail view that displays an active user profile by username.
        
    """

    queryset = get_user_model().objects.get_visible_profiles()
    template_name = "accounts/profile_detail.html"
    slug_field = 'username'
    slug_url_kwarg = 'username'
    
    


# -*- coding: utf-8 -*-
from django import forms
from django import template
from django import VERSION as DJANGO_VERSION

from django.template import Context
from django.template.loader import get_template
from manifest.bootstrap import defaults

if DJANGO_VERSION >= (1, 10, 0):
    context_class = dict
else:
    # Django<1.10 compatibility
    from django.template import Context
    context_class = Context

register = template.Library()


@register.filter
def bootstrap(element):
    markup_classes = {'label': '', 'value': '', 'single_value': ''}
    return render(element, markup_classes)


@register.filter
def bootstrap_inline(element):
    markup_classes = {'label': 'sr-only', 'value': '', 'single_value': ''}
    return render(element, markup_classes)


@register.filter
def bootstrap_horizontal(element, label_cols={}):
    if not label_cols:
        label_cols = 'col-sm-2 col-lg-2'

    markup_classes = {'label': label_cols,
            'value': '',
            'single_value': ''}

    for cl in label_cols.split(' '):
        splited_class = cl.split('-')

        try:
            value_nb_cols = int(splited_class[-1])
        except ValueError:
            value_nb_cols = defaults.BOOTSTRAP_GRID_COLUMNS

        if value_nb_cols >= defaults.BOOTSTRAP_GRID_COLUMNS:
            splited_class[-1] = defaults.BOOTSTRAP_GRID_COLUMNS
        else:
            offset_class = cl.split('-')
            offset_class[-1] = 'offset-' + str(value_nb_cols)
            splited_class[-1] = str(defaults.BOOTSTRAP_GRID_COLUMNS - value_nb_cols)
            markup_classes['single_value'] += ' ' + '-'.join(offset_class)
            markup_classes['single_value'] += ' ' + '-'.join(splited_class)

        markup_classes['value'] += ' ' + '-'.join(splited_class)

    return render(element, markup_classes)


def add_input_classes(field):
    if not is_checkbox(field) and not is_multiple_checkbox(field) and not is_radio(field) \
        and not is_file(field):
        field_classes = field.field.widget.attrs.get('class', '')
        field_classes += ' form-control'
        field.field.widget.attrs['class'] = field_classes


def render(element, markup_classes):
    element_type = element.__class__.__name__.lower()

    if element_type == 'boundfield':
        add_input_classes(element)
        template = get_template("bootstrap/field.html")
        context = context_class({'field': element, 'classes': markup_classes})
    else:
        has_management = getattr(element, 'management_form', None)
        if has_management:
            for form in element.forms:
                for field in form.visible_fields():
                    add_input_classes(field)

            template = get_template("bootstrap/formset.html")
            context = context_class({'formset': element, 'classes': markup_classes})
        else:
            for field in element.visible_fields():
                add_input_classes(field)

            template = get_template("bootstrap/form.html")
            context = context_class({'form': element, 'classes': markup_classes})

    return template.render(context)


@register.filter
def is_checkbox(field):
    return isinstance(field.field.widget, forms.CheckboxInput)


@register.filter
def is_multiple_checkbox(field):
    return isinstance(field.field.widget, forms.CheckboxSelectMultiple)


@register.filter
def is_radio(field):
    return isinstance(field.field.widget, forms.RadioSelect)


@register.filter
def is_file(field):
    return isinstance(field.field.widget, forms.FileInput)


@register.filter
def css(field, css):
    return field.as_widget(attrs={"class":css})


# -*- coding: utf-8 -*-
from django.core.management import call_command
from django.contrib.auth import get_user_model

from manifest.accounts import defaults
from manifest.accounts.tests.base import AccountsTestCase

import datetime


class CleanExpiredTests(AccountsTestCase):
    user_info = {'username': 'alice',
                 'password': 'swordfish',
                 'email': 'alice@example.com'}

    def test_clean_expired(self):
        """
        Test if ``clean_expired`` deletes all users which ``activation_key``
        is expired.

        """
        # Create an account which is expired.
        user = get_user_model().objects.create_user(**self.user_info)
        user.date_joined -= datetime.timedelta(days=defaults.ACCOUNTS_ACTIVATION_DAYS + 1)
        user.save()

        # There should be one account now
        get_user_model().objects.get(username=self.user_info['username'])

        # Clean it.
        call_command('clean_expired')

        self.failUnlessEqual(get_user_model().objects.filter(username=self.user_info['username']).count(), 0)



# -*- coding: utf-8 -*-
import hashlib

from django.conf import settings
from django.contrib.auth import get_user_model

from manifest.accounts.utils import (get_gravatar, login_redirect,
                           get_protocol)
from manifest.accounts import defaults
from manifest.accounts.tests.base import AccountsTestCase

class UtilsTests(AccountsTestCase):
    """ Test the extra utils methods """
    fixtures = ['test']

    def test_get_gravatar(self):
        template = 'http://www.gravatar.com/avatar/%(hash)s?s=%(size)s&d=%(type)s'

        # The hash for alice@example.com
        hash = hashlib.md5('alice@example.com'.encode('utf-8')).hexdigest()

        # Check the defaults.
        self.failUnlessEqual(get_gravatar('alice@example.com'),
                             template % {'hash': hash,
                                         'size': 80,
                                         'type': 'identicon'})

        # Check different size
        self.failUnlessEqual(get_gravatar('alice@example.com', size=200),
                             template % {'hash': hash,
                                         'size': 200,
                                         'type': 'identicon'})

        # Check different default
        http_404 = get_gravatar('alice@example.com', default='404')
        self.failUnlessEqual(http_404,
                             template % {'hash': hash,
                                         'size': 80,
                                         'type': '404'})

        # Is it really a 404?
        response = self.client.get(http_404)
        self.failUnlessEqual(response.status_code, 404)

        # Test the switch to HTTPS
        defaults.ACCOUNTS_GRAVATAR_SECURE = True

        template = 'https://secure.gravatar.com/avatar/%(hash)s?s=%(size)s&d=%(type)s'
        self.failUnlessEqual(get_gravatar('alice@example.com'),
                             template % {'hash': hash,
                                         'size': 80,
                                         'type': 'identicon'})

        # And set back to default
        defaults.ACCOUNTS_GRAVATAR_SECURE = False

    def test_login_redirect(self):
        """
        Test redirect function which should redirect the user after a
        succesfull signin.

        """
        # Test with a requested redirect
        self.failUnlessEqual(login_redirect(redirect='/accounts/'), '/accounts/')

        # Test with only the user specified
        user = get_user_model().objects.get(pk=1)
        self.failUnlessEqual(login_redirect(user=user),
                             '/profiles/%s/' % user.username)
        
        # The ultimate fallback, probably never used
        self.failUnlessEqual(login_redirect(), settings.LOGIN_REDIRECT_URL)

    def test_get_protocol(self):
        """ Test if the correct protocol is returned """
        self.assertEqual(get_protocol(), 'http')
        with self.settings(ACCOUNTS_USE_HTTPS=True):
            self.assertEqual(get_protocol(), 'https')



from django.conf import settings
from django.test import TestCase
from django.utils.translation import activate, deactivate

class AccountsTestCase(TestCase):
    """ A custom TestCase that loads the accounts application for testing purposes """
    def _pre_setup(self):
        activate(settings.LANGUAGE_CODE)
        super(AccountsTestCase, self)._pre_setup()

    def _post_teardown(self):
        super(AccountsTestCase, self)._post_teardown()
        deactivate()
        


# -*- coding: utf-8 -*-
from django.conf import settings
from django.http import HttpRequest
from importlib import import_module
from django.contrib.auth import get_user_model
from django.core.exceptions import ObjectDoesNotExist

from manifest.accounts.tests.base import AccountsTestCase
from manifest.accounts.middleware import LocaleMiddleware
from manifest.accounts import defaults

class LocaleMiddlewareTests(AccountsTestCase):
    """ Test the ``LocaleMiddleware`` """
    fixtures = ['test']

    def _get_request_with_user(self, user):
        """ Fake a request with an user """
        request = HttpRequest()
        request.META = {
            'SERVER_NAME': 'testserver',
            'SERVER_PORT': 80,
        }
        request.method = 'GET'
        request.session = {}

        # Add user
        request.user = user
        return request

    def test_preference_user(self):
        """ Test the language preference of two users """
        users = ((1, 'tr-tr'),
                 (2, 'en-us'))

        for pk, lang in users:
            user = get_user_model().objects.get(pk=pk)

            req = self._get_request_with_user(user)

            # Check that the user has this preference
            self.failUnlessEqual(user.locale, lang)

            # Request should have a ``LANGUAGE_CODE`` with dutch
            LocaleMiddleware().process_request(req)
            self.failUnlessEqual(req.LANGUAGE_CODE, lang)

    def test_without_language_field(self):
        """ Middleware should do nothing if the profile has no language field """
        defaults.ACCOUNTS_LOCALE_FIELD = 'non_existant_language_field'
        user = get_user_model().objects.get(pk=1)

        req = self._get_request_with_user(user)

        # Middleware should do nothing
        LocaleMiddleware().process_request(req)
        self.failIf(hasattr(req, 'LANGUAGE_CODE'))


# -*- coding: utf-8 -*-
from django.utils import translation
from django.core.exceptions import ObjectDoesNotExist
from django.conf import settings

from manifest.accounts import defaults

class LocaleMiddleware(object):
    """
    Set the language by looking at the language setting in the profile.

    It doesn't override the cookie that is set by Django so a user can still
    switch languages depending if the cookie is set.

    """
    def process_request(self, request):
        lang_cookie = request.session.get(settings.LANGUAGE_COOKIE_NAME)
        if not lang_cookie:
            if request.user.is_authenticated:
                try:
                    lang = getattr(request.user, 
                                defaults.ACCOUNTS_LOCALE_FIELD)
                    translation.activate(lang)
                    request.LANGUAGE_CODE = translation.get_language()
                except AttributeError: pass



# -*- coding: utf-8 -*-
"""
Django settings for {{ project_name }} project.

For more information on this file, see
https://docs.djangoproject.com/en/{{ docs_version }}/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/{{ docs_version }}/ref/settings/
"""

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
import os
BASE_DIR = os.path.dirname(os.path.dirname(__file__))
BASE_URL = '/'


# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/{{ docs_version }}/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = '{{ secret_key }}'

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

TEMPLATE_DEBUG = DEBUG
THUMBNAIL_DEBUG = DEBUG

ALLOWED_HOSTS = []


# Application definition

INSTALLED_APPS = (
    # Django Contrib Apps
    'django.contrib.admin',
    'django.contrib.admindocs',
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.flatpages',
    'django.contrib.humanize',	    
    'django.contrib.messages',    
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.staticfiles',

    # Django Manifest Apps
    'manifest.core',
    'manifest.accounts',
    'manifest.bootstrap',
)

MIDDLEWARE = (
    'django.middleware.security.SecurityMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    # Enable to use browsers locale settings
    # 'django.middleware.locale.LocaleMiddleware',
    'django.middleware.common.CommonMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'django.middleware.clickjacking.XFrameOptionsMiddleware',
    'django.contrib.flatpages.middleware.FlatpageFallbackMiddleware',
)

ROOT_URLCONF = '{{ project_name }}.urls'

# Python dotted path to the WSGI application used by Django's runserver.
WSGI_APPLICATION = '{{ project_name }}.wsgi.application'


# Database
# https://docs.djangoproject.com/en/{{ docs_version }}/ref/settings/#databases

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',             # Add 'postgresql_psycopg2', 'postgresql', 'mysql', 'sqlite3' or 'oracle'.
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),       # Or path to database file if using sqlite3.
        # The following settings are not used with sqlite3:
        'USER': '',                         # Not used with sqlite3.
        'PASSWORD': '',                     # Not used with sqlite3.
        'HOST': '',                         # Set to empty string for localhost. Not used with sqlite3.
        'PORT': '',                         # Set to empty string for default. Not used with sqlite3.
    }
}


# Internationalization
# https://docs.djangoproject.com/en/{{ docs_version }}/topics/i18n/

TIME_ZONE = 'UTC' #'Europe/Istanbul'
USE_TZ = True

USE_I18N = True
USE_L10N = True

LANGUAGE_CODE = 'en-us' #'tr-tr'

ugettext = lambda s: s
# Languages Django Manifest provide translations for, out of the box.
LANGUAGES = (
  ('en', ugettext('English')),
  ('tr', ugettext('Turkish')),
)

TEST_RUNNER = 'django.test.runner.DiscoverRunner'

# Project specific configuration
SITE_ID = 1

# SITE_URL needed if project will not be served at domain root
SITE_URL = BASE_URL


# People who get code error notifications.
# In the format (('Full Name', 'email@example.com'), ('Full Name', 'anotheremail@example.com')
ADMINS = (
	# ('Your Name', 'your_email@example.com'),
)

# Not-necessarily-technical managers of the site. They get broken link
# notifications and other various emails.
MANAGERS = ADMINS

# Absolute filesystem path to the directory that will hold user-uploaded files.
# Example: "/var/www/example.com/media/"
MEDIA_ROOT = os.path.join(BASE_DIR, 'media/')

# URL that handles the media served from MEDIA_ROOT.
# Examples: "http://example.com/media/", "http://media.example.com/"
MEDIA_URL = os.path.join(BASE_URL, 'media/')

# Absolute path to the directory static files should be collected to.
# Example: "/var/www/example.com/static/"
STATIC_ROOT = os.path.join(BASE_DIR, 'static/')

# URL that handles the static files served from STATIC_ROOT.
# Example: "http://example.com/static/", "http://static.example.com/"
STATIC_URL = os.path.join(BASE_URL, 'static/')

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [os.path.join(BASE_DIR, 'templates'),],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': [
                'django.template.context_processors.debug',
                'django.template.context_processors.request',
                'django.contrib.auth.context_processors.auth',
                'django.contrib.messages.context_processors.messages',
            ],
            'builtins': [
                'django.templatetags.i18n', 
                'manifest.bootstrap.templatetags.bootstrap_tags'
            ],
        },
    },
]


# User registration & authentication
AUTHENTICATION_BACKENDS = (
    # Django Contrib
    'django.contrib.auth.backends.ModelBackend',
    # Django Manifest
    'manifest.accounts.backends.AuthenticationBackend',
)

# Django Auth Settings
AUTH_USER_MODEL = 'accounts.User'

# The email backend to use.
EMAIL_BACKEND = 'django.core.mail.backends.smtp.EmailBackend'

# Host for sending email.
EMAIL_HOST = '' 
EMAIL_PORT = 587

# SMTP Authentication
EMAIL_HOST_USER = ''
EMAIL_HOST_PASSWORD = ''
EMAIL_USE_TLS = True
EMAIL_USE_SSL = False

# Automated correspondence
DEFAULT_FROM_EMAIL = 'Django Manifest <django.manifest@localhost>'
EMAIL_SUBJECT_PREFIX = '[Django Manifest] '

# local_settings.py can be used to override environment-specific settings
# like database and email that differ between development and production.
try:
    from .settings_local import *
except ImportError:
    pass
    


# -*- coding: utf-8 -*-
"""
Django local settings for {{ project_name }} project.
"""

import os
from . import settings

BASE_DIR = os.path.dirname(os.path.dirname(__file__))

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.sqlite3',
        'NAME': os.path.join(BASE_DIR, 'db.sqlite3'),
    }
}

MIDDLEWARE = settings.MIDDLEWARE + (
    'debug_toolbar.middleware.DebugToolbarMiddleware',
)

INSTALLED_APPS = settings.INSTALLED_APPS + (
    'debug_toolbar',
)

# Settings for Django Debug Toolbar
INTERNAL_IPS = ('127.0.0.1',)
DEBUG_TOOLBAR_CONFIG = {
#    'INTERCEPT_REDIRECTS': False,
#    'SHOW_TOOLBAR_CALLBACK': custom_show_toolbar,
#    'EXTRA_SIGNALS': ['{{ project_name }}.signals.MySignal'],
#    'HIDE_DJANGO_SQL': False,
#    'TAG': 'div',
#    'ENABLE_STACKTRACES' : True,
}

# The email backend to use.
EMAIL_BACKEND = 'django.core.mail.backends.console.EmailBackend'

# Host for sending email.
# EMAIL_HOST = '' 
# EMAIL_PORT = 587

# SMTP Authentication
# EMAIL_HOST_USER = ''
# EMAIL_HOST_PASSWORD = ''
# EMAIL_USE_TLS = True
# EMAIL_USE_SSL = False



"""backend URL Configuration

The `urlpatterns` list routes URLs to views. For more information please see:
    https://docs.djangoproject.com/en/2.1/topics/http/urls/
Examples:
Function views
    1. Add an import:  from my_app import views
    2. Add a URL to urlpatterns:  path('', views.home, name='home')
Class-based views
    1. Add an import:  from other_app.views import Home
    2. Add a URL to urlpatterns:  path('', Home.as_view(), name='home')
Including another URLconf
    1. Import the include() function: from django.urls import include, path
    2. Add a URL to urlpatterns:  path('blog/', include('blog.urls'))
"""
from django.conf import settings
from django.contrib import admin
from django.urls import path, re_path, include
from django.conf.urls.static import static

from django.views.generic import TemplateView


urlpatterns = [
    path('admin/', admin.site.urls),
    
    # Django Manifest standard homepage:
    re_path(r'^$', TemplateView.as_view(template_name='homepage.html'), name='home'),

    re_path(r'^', include('manifest.accounts.urls')),
    ]

if settings.DEBUG:
    import debug_toolbar
    urlpatterns += [path('__debug__/', include(debug_toolbar.urls)),]
    urlpatterns += static(settings.MEDIA_URL, document_root=settings.MEDIA_ROOT)



# -*- coding: utf-8 -*-
from django import template
from django.conf import settings

register = template.Library()

@register.inclusion_tag("core/analytics.html")
def analytics(account=None, *args, **kwargs):
    """
    Simple Google Analytics integration.

    First looks for an ``account`` parameter. If not supplied, uses
    Django ``GOOGLE_ANALYTICS_ACCOUNT`` setting. If account not set, 
    raises ``TemplateSyntaxError``.

    :param account:
        Google Analytics account id to be used.

    """
    if not account:
        try:
            account = settings.GOOGLE_ANALYTICS_ACCOUNT
        except:
            raise template.TemplateSyntaxError( 
                    "Analytics account could not found either "
                    "in tag parameters or settings")
    return {'account': account, 'params':kwargs }



# -*- coding: utf-8 -*-
from django.conf import settings
from django.contrib.sites.models import Site

def site(request):
    site = Site.objects.get_current()
    return {
        'SITE_NAME': getattr(settings, 'SITE_NAME', site.name),
        'SITE_DOMAIN': getattr(settings, 'SITE_DOMAIN', site.domain),
        'SITE_PATH': getattr(settings, 'SITE_PATH', ''),
        'SITE_URL': getattr(settings, 'SITE_URL', ''),
    }

def installed_apps(request):
    return {'INSTALLED_APPS': getattr(settings, 'INSTALLED_APPS', None)}

def remote_addr(request):
    return {'REMOTE_ADDR': request.META['REMOTE_ADDR']}



# -*- coding: utf-8 -*-
from django.urls import path, re_path, include
from django.contrib.auth import views as auth_views
from django.contrib.auth.decorators import login_required

from manifest.accounts import views as accounts_views
from manifest.accounts import defaults


urlpatterns = [
    # Signup, login and logout
    re_path(r'^login/$', 
        accounts_views.Login.as_view(),
        name='accounts_login'),

    re_path(r'^logout/$', 
        auth_views.LogoutView.as_view(
            template_name='accounts/logout.html'
            ),
        name='accounts_logout'), 

    re_path(r'^register/$', 
        accounts_views.Register.as_view(),
        name='accounts_register'),

    re_path(r'^register/complete/(?P<username>\w+)/$', 
        accounts_views.UserView.as_view(
            template_name='accounts/register_complete.html',
            extra_context={
                'accounts_activation_required': 
                    defaults.ACCOUNTS_ACTIVATION_REQUIRED,
                'accounts_activation_days': 
                    defaults.ACCOUNTS_ACTIVATION_DAYS}),
        name='accounts_register_complete'),

    # Activate
    re_path(r'^activate/(?P<username>\w+)/(?P<activation_key>\w+)/$', 
        accounts_views.Activate.as_view(), 
        name='accounts_activate'),

    # Disabled
    re_path(r'^disabled/(?P<username>\w+)/$', 
        accounts_views.UserView.as_view(
            template_name='accounts/disabled.html'), 
        name='accounts_disabled'),

    # Settings
    re_path(r'^settings/$', 
        accounts_views.UserTemplateView.as_view(),
        name='accounts_settings'),

    # Edit profile
    re_path(r'^settings/update/$', 
        accounts_views.ProfileUpdate.as_view(),    
        name='accounts_update'),

    # Reset password using django.contrib.auth.views
    re_path(r'^password/reset/$', 
        auth_views.PasswordResetView.as_view(
            template_name='accounts/password_reset_form.html',
            email_template_name='accounts/emails/password_reset_message.txt'
            ),
        name='accounts_password_reset'),

    re_path(r'^password/reset/done/$', 
        auth_views.PasswordResetDoneView.as_view(
            template_name='accounts/password_reset_done.html'
            ),
        name='password_reset_done'),

    re_path(r'^password/reset/confirm/(?P<uidb64>[0-9A-Za-z_\-]+)/(?P<token>.+)/$', 
        auth_views.PasswordResetConfirmView.as_view(
            template_name='accounts/password_reset_confirm.html'
            ),
        name='password_reset_confirm'),

    re_path(r'^password/reset/complete/$', 
        auth_views.PasswordResetCompleteView.as_view(
            template_name='accounts/password_reset_complete.html'
            ),
        name='password_reset_complete'),

    # Change email and confirm it
    re_path(r'^email/change/$', 
        accounts_views.EmailChange.as_view(),
        name='accounts_email_change'),

    re_path(r'^email/change/done/(?P<username>\w+)/$', 
        accounts_views.UserView.as_view(
            template_name='accounts/email_change_done.html'),
        name='accounts_email_change_done'),
    
    re_path(r'^email/change/confirm/(?P<username>\w+)/(?P<confirmation_key>\w+)/$', 
        accounts_views.EmailConfirm.as_view(),
        name='accounts_email_confirm'),

    re_path(r'^email/change/complete/(?P<username>\w+)/$', 
        accounts_views.UserView.as_view(
            template_name='accounts/email_change_complete.html'),
        name='accounts_email_change_complete'),

    # Change password
    re_path(r'^password/change/$', 
        accounts_views.PasswordChange.as_view(),
        name='accounts_password_change'),

    re_path(r'^password/change/done/(?P<username>\w+)/$', 
        accounts_views.UserView.as_view(
            template_name='accounts/password_change_complete.html'),
        name='accounts_password_change_done'),

    # View profiles
    re_path(r'^profiles/(?P<username>(?!logout|register|login|password|account|profile)\w+)/$', 
        accounts_views.ProfileDetail.as_view(),
        name='accounts_profile_detail'),

    re_path(r'^profiles/$', 
        accounts_views.ProfileList.as_view(),
        name='accounts_profile_list'),
]


# -*- coding: utf-8 -*-
import re
import hashlib
import datetime

from django.core import mail
from django.conf import settings

from django.contrib.sites.models import Site
from django.contrib.auth import get_user_model
from django.contrib.auth.models import AnonymousUser

from manifest.accounts import defaults
from manifest.accounts.models import upload_to_picture
from manifest.accounts.tests.base import AccountsTestCase


class AccountModelTests(AccountsTestCase):
    """ Test the model of Account """
    user_info = {'username': 'foo',
                 'password': 'bar',
                 'email': 'foo@example.com'}

    fixtures = ['test']

    def test_change_email(self):
        """ TODO """
        pass

    def test_activation_expired_account(self):
        """
        ``Account.activation_key_expired()`` is ``True`` when the
        ``activation_key_created`` is more days ago than defined in
        ``ACCOUNTS_ACTIVATION_DAYS``.

        """
        user = get_user_model().objects.create_user(**self.user_info)
        user.date_joined -= datetime.timedelta(days=defaults.ACCOUNTS_ACTIVATION_DAYS + 1)
        user.save()

        user = get_user_model().objects.get(username='foo')
        self.failUnless(user.activation_key_expired())

    def test_activation_used_account(self):
        """
        An user cannot be activated anymore once the activation key is
        already used.

        """
        user = get_user_model().objects.create_user(**self.user_info)
        activated_user = get_user_model().objects.activate_user(user.username,
                                                             user.activation_key)
        self.failUnless(activated_user.activation_key_expired())

    def test_activation_unexpired_account(self):
        """
        ``Account.activation_key_expired()`` is ``False`` when the
        ``activation_key_created`` is within the defined timeframe.``

        """
        user = get_user_model().objects.create_user(**self.user_info)
        self.failIf(user.activation_key_expired())

    def test_activation_email(self):
        """
        When a new account is created, a activation e-mail should be send out
        by ``Account.send_activation_email``.

        """
        new_user = get_user_model().objects.create_user(**self.user_info)
        self.failUnlessEqual(len(mail.outbox), 1)
        self.assertEqual(mail.outbox[0].to, [self.user_info['email']])


    def test_upload_picture(self):
        """
        Test the uploaded path of pictures

        TODO: What if a image get's uploaded with no extension?

        """
        user = get_user_model().objects.get(pk=1)
        filename = 'my_avatar.png'
        path = upload_to_picture(user, filename)

        # Path should be changed from the original
        self.failIfEqual(filename, path)

        # Check if the correct path is returned
        PICTURE_RE = re.compile('^%(picture_path)s/[a-f0-9]{10}.png$' %
                            {'picture_path': getattr(defaults, 
                                'ACCOUNTS_PICTURE_PATH','%s/%s' % (
                                    str(user._meta.app_label), 
                                    str(user._meta.model_name)))})

        self.failUnless(PICTURE_RE.search(path))

    def test_picture_url(self):
        """ The user has uploaded it's own picture. This should be returned. """
        filename = 'fake_image.png'
        user = get_user_model().objects.get(pk=1)
        user.picture = filename
        user.save()
        self.failUnlessEqual(user.get_picture_url(),
                             settings.MEDIA_URL + filename)

    def test_get_picture_url_without_gravatar(self):
        """
        Test if the correct picture is returned for the user when
        ``ACCOUNTS_PICTURE_GRAVATAR`` is set to ``False``.

        """
        user = get_user_model().objects.get(pk=1)
        # This user has no picture, and gravatar is disabled. And to make
        # matters worse, there isn't even a default image.
        defaults.ACCOUNTS_GRAVATAR_PICTURE = False
        self.failUnlessEqual(user.get_picture_url(), None)

        # There _is_ a default image
        defaults.ACCOUNTS_GRAVATAR_DEFAULT = 'http://example.com'
        self.failUnlessEqual(user.get_picture_url(), 'http://example.com')

        # Settings back to default
        defaults.ACCOUNTS_PICTURE_GRAVATAR = True

    def test_get_picture_url_with_gravatar(self):
        """
        Test if the correct picture is returned when the user makes use of gravatar.

        """
        template = 'http://www.gravatar.com/avatar/%(hash)s?s=%(size)s&d=%(default)s'
        user = get_user_model().objects.get(pk=1)

        gravatar_hash = hashlib.md5(user.email.encode('utf-8')).hexdigest()

        # Test with the default settings
        self.failUnlessEqual(user.get_picture_url(),
                             template % {'hash': gravatar_hash,
                                         'size': defaults.ACCOUNTS_GRAVATAR_SIZE,
                                         'default': defaults.ACCOUNTS_GRAVATAR_DEFAULT})

        # Change accounts settings
        defaults.ACCOUNTS_GRAVATAR_SIZE = 180
        defaults.ACCOUNTS_GRAVATAR_DEFAULT = '404'

        self.failUnlessEqual(user.get_picture_url(),
                             template % {'hash': gravatar_hash,
                                         'size': defaults.ACCOUNTS_GRAVATAR_SIZE,
                                         'default': defaults.ACCOUNTS_GRAVATAR_DEFAULT})

        # Settings back to default
        defaults.ACCOUNTS_PICTURE_MAX_SIZE = 80
        defaults.ACCOUNTS_GRAVATAR_DEFAULT = 'identicon'

    def test_get_full_name_or_username(self):
        """ Test if the full name or username are returned correcly """
        user = get_user_model().objects.get(pk=1)

        # Profile #1 has a first and last name
        full_name = user.get_full_name_or_username()
        self.failUnlessEqual(full_name, "John Doe")

        # Let's empty out his name, now we should get his username
        user.first_name = ''
        user.last_name = ''
        user.save()

        self.failUnlessEqual(user.get_full_name_or_username(),
                             "john")

        # Finally, accounts doesn't use any usernames, so we should return the
        # e-mail address.
        defaults.ACCOUNTS_WITHOUT_USERNAMES = True
        self.failUnlessEqual(user.get_full_name_or_username(),
                             "john@example.com")
        defaults.ACCOUNTS_WITHOUT_USERNAMES = False



# -*- coding: utf-8 -*-
# Accounts settings file.
#
# Please consult the docs for more information about each setting.

from django.conf import settings
gettext = lambda s: s


ACCOUNTS_REDIRECT_ON_LOGOUT = getattr(settings,
                                'ACCOUNTS_REDIRECT_ON_LOGOUT',
                                None)

ACCOUNTS_LOGIN_REDIRECT_URL = '/profiles/%(username)s/' # getattr(settings, 
                              #   'LOGIN_REDIRECT_URL', 
                              #   '/profiles/%(username)s/')

ACCOUNTS_ACTIVATION_REQUIRED = getattr(settings,
                                'ACCOUNTS_ACTIVATION_REQUIRED',
                                True)

ACCOUNTS_ACTIVATION_DAYS = getattr(settings,
                                'ACCOUNTS_ACTIVATION_DAYS',
                                7)

ACCOUNTS_ACTIVATED = getattr(settings,
                        'ACCOUNTS_ACTIVATED',
                        'ALREADY_ACTIVATED')

ACCOUNTS_REMEMBER_ME_DAYS = getattr(settings,
                                'ACCOUNTS_REMEMBER_ME_DAYS',
                                (gettext('a month'), 30))

ACCOUNTS_FORBIDDEN_USERNAMES = getattr(settings,
                                    'ACCOUNTS_FORBIDDEN_USERNAMES',
                                    ('register', 'activate', 'logout', 
                                        'login', 'me', 'user',  'password', 
                                        'account', 'profile', 'about', 
                                        'contact'))

ACCOUNTS_USE_HTTPS = getattr(settings,
                        'ACCOUNTS_USE_HTTPS',
                        False)

ACCOUNTS_PICTURE_MAX_SIZE = getattr(settings,
                               'ACCOUNTS_PICTURE_MAX_SIZE',
                               '1024 x 1024')

ACCOUNTS_PICTURE_PATH = getattr(settings,
                           'ACCOUNTS_PICTURE_PATH',
                           'accounts')

ACCOUNTS_PICTURE_FORMATS = getattr(settings,
                               'ACCOUNTS_PICTURE_FORMATS',
                               ['jpeg', 'gif', 'png'])

ACCOUNTS_PICTURE_MAX_FILE = getattr(settings,
                               'ACCOUNTS_PICTURE_MAX_FILE',
                               1024*1024)

ACCOUNTS_GRAVATAR_MAX_FILE = getattr(settings,
                               'ACCOUNTS_PICTURE_MAX_FILE',
                               1024*1024)

ACCOUNTS_GRAVATAR_PICTURE = getattr(settings,
                               'ACCOUNTS_GRAVATAR_PICTURE',
                               True)

ACCOUNTS_GRAVATAR_SECURE = getattr(settings,
                              'ACCOUNTS_GRAVATAR_SECURE',
                              ACCOUNTS_USE_HTTPS)

ACCOUNTS_GRAVATAR_DEFAULT = getattr(settings,
                              'ACCOUNTS_GRAVATAR_DEFAULT',
                              'identicon')

ACCOUNTS_GRAVATAR_SIZE = getattr(settings,
                           'ACCOUNTS_GRAVATAR_SIZE',
                           128)

ACCOUNTS_DISABLE_PROFILE_LIST = getattr(settings,
                                   'ACCOUNTS_DISABLE_PROFILE_LIST',
                                   False)

ACCOUNTS_USE_MESSAGES = getattr(settings,
                           'ACCOUNTS_USE_MESSAGES',
                           False)

ACCOUNTS_LOCALE_FIELD = getattr(settings,
                             'ACCOUNTS_LOCALE_FIELD',
                             'locale')

ACCOUNTS_WITHOUT_USERNAMES = getattr(settings,
                                'ACCOUNTS_WITHOUT_USERNAMES',
                                False)

ACCOUNTS_HIDE_EMAIL = getattr(settings, 
                            'ACCOUNTS_HIDE_EMAIL', 
                            True)



# -*- coding: utf-8 -*-
import re, datetime
from django.db import models
from django.db.models import Q
from django.contrib.auth import get_user_model
from django.contrib.auth.models import UserManager as BaseManager, AnonymousUser
from django.contrib.contenttypes.models import ContentType
from django.utils.translation import ugettext as _

from manifest.accounts import defaults
from manifest.accounts import signals as accounts_signals
from manifest.accounts.utils import generate_sha1


SHA1_RE = re.compile('^[a-f0-9]{40}$')


class AccountActivationManager(BaseManager):
    """
    Registration and account activation functionalities for User model.
    """
    
    def create_user(self, username, email, password, active=False,
                    send_email=True):
        """
        A simple wrapper that creates a new :class:`User`.

        :param username:
            String containing the username of the new user.

        :param email:
            String containing the email address of the new user.

        :param password:
            String containing the password for the new user.

        :param active:
            Boolean that defines if the user requires activation by clicking 
            on a link in an email. Defauts to ``True``.

        :param send_email:
            Boolean that defines if the user should be send an email. You 
            could set this to ``False`` when you want to create a user in 
            your own code, but don't want the user to activate through email.

        :return: :class:`User` instance representing the new user.

        """

        user = super(AccountActivationManager, self).create_user(username, email, password)

        if isinstance(user.username, str):
            username = user.username.encode('utf-8')
        salt, activation_key = generate_sha1(username)
        user.is_active = active
        user.activation_key = activation_key
        user.save(using=self._db)

        if send_email:
            user.send_activation_email()
 
        return user

    def activate_user(self, username, activation_key):
        """
        Activate an :class:`User` by supplying a valid ``activation_key``.

        If the key is valid and an user is found, activates the user and
        return it. Also sends the ``activation_complete`` signal.

        :param activation_key:
            String containing the secret SHA1 for a valid activation.

        :return:
            The newly activated :class:`User` or ``False`` if not successful.

        """
        if SHA1_RE.search(activation_key):
            try:
                user = self.get(activation_key=activation_key)
            except self.model.DoesNotExist:
                return False
            if not user.activation_key_expired():
                user.activation_key = defaults.ACCOUNTS_ACTIVATED
                user.is_active = True
                user.save(using=self._db)
                # Send the activation_complete signal
                accounts_signals.activation_complete.send(sender=None, 
                    user=user)
                return user
        return False

    def delete_expired_users(self):
        """
        Checks for expired users and delete's the ``User`` associated with
        it. Skips if the user ``is_staff``.

        :return: A list containing the deleted users.

        """
        deleted_users = []
        for user in self.filter(is_staff=False, is_active=False):
            if user.activation_key_expired():
                deleted_users.append(user)
                user.delete()
        return deleted_users    
    
class EmailConfirmationManager(BaseManager):
    """
    E-mail address confirmation functionalities for User model.
    """
    
    def confirm_email(self, username, confirmation_key):
        """
        Confirm an email address by checking a ``confirmation_key``.

        A valid ``confirmation_key`` will set the newly wanted email address
        as the current email address. Returns the user after success or
        ``False`` when the confirmation key is invalid.

        :param confirmation_key:
            String containing the secret SHA1 that is used for verification.

        :return:
            The verified :class:`User` or ``False`` if not successful.

        """
        if SHA1_RE.search(confirmation_key):
            try:
                user = self.select_related().get(username=username,
                                    email_confirmation_key=confirmation_key,
                                    email_unconfirmed__isnull=False)
            except self.model.DoesNotExist:
                return False
            else:
                user.email = user.email_unconfirmed
                user.email_unconfirmed, user.email_confirmation_key = '',''
                user.save(using=self._db)
                # Send the confirmation_complete signal
                accounts_signals.confirmation_complete.send(sender=None, 
                    user=user)
                return user
        return False


class UserProfileManager(BaseManager):
    """
    Profile functionalities for User model.
    """

    def get_visible_profiles(self, user=None):
        """
        Returns all the visible profiles available to this user.

        For now keeps it simple by just applying the cases when a user is not
        active, a user has it's profile closed to everyone or a user only
        allows registered users to view their profile.

        :param user:
            A Django :class:`User` instance.

        :return:
            All profiles that are visible to this user.

        """
        profiles = self.select_related().all()

        filter_kwargs = {'is_active': True}

        profiles = profiles.filter(**filter_kwargs)
        if user and isinstance(user, AnonymousUser):
            profiles = []
        return profiles


class BaseUserManager(AccountActivationManager,
                        EmailConfirmationManager,
                        UserProfileManager):
    pass



class UserManager(BaseUserManager):
    """ Extra functionality for the User model. """
    pass


# -*- coding: utf-8 -*-
try:
    from django.core.management.base import NoArgsCommand as BaseCommand
except ImportError:
    from django.core.management.base import BaseCommand
    
from django.contrib.auth import get_user_model


class Command(BaseCommand):
    """
    Search for users that still haven't verified their email after
    ``ACCOUNTS_ACTIVATION_DAYS`` and delete them.

    """
    help = 'Deletes expired users.'
    def handle(self, *args, **options):
        users = get_user_model().objects.delete_expired_users()



# -*- coding: utf-8 -*-
import re
from django import template
from django.urls import reverse, NoReverseMatch, resolve, Resolver404

register = template.Library()

@register.simple_tag(takes_context=True)
def navactive(request, url, exact=1, use_resolver=1):
    """
    Returns ``active`` if the given URL is in the url path, otherwise ''.
    Usage::
        {% load libs_tags %}
        ...
        <li class="{% navactive request "/news/" exact=1 %}">
    :param request: A request instance.
    :param url: A string representing a part of the URL that needs to exist
      in order for this method to return ``True``.
    :param exact: If ``1`` then the parameter ``url`` must be equal to
      ``request.path``, otherwise the parameter ``url`` can just be a part of
      ``request.path``.
    :use_resolver: If ``0`` we will not try to compare ``url`` with existing
      view names but we will only compare it with ``request.path``.
    """
    if use_resolver:
        try:
            if url == resolve(request.path).url_name:
                # Checks the url pattern in case a view_name is posted
                return 'active'
            elif url == request.path:
                # Workaround to catch URLs with more than one part, which don't
                # raise a Resolver404 (e.g. '/index/info/')
                match = request.path
            else:
                return ''
        except Resolver404:
            # Indicates, that a simple url string is used (e.g. '/index/')
            match = request.path
    else:
        match = request.path

    if exact and url == match:
        return 'active'
    elif not exact and url in request.path:
        return 'active'
    return ''

@register.simple_tag(takes_context=True)
def active(context, pattern_or_urlname):
    try:
        pattern = '^' + reverse(pattern_or_urlname)
    except NoReverseMatch:
        pattern = pattern_or_urlname
    path = context['request'].path
    if re.search(pattern, path):
        return 'active'
    return ''
    

@register.simple_tag(takes_context=True)
def active_url(context, urls, css=None):
    """
    Highlight menu item based on url tag.
    
    Returns a css class if ``request.path`` is in given ``url``.
    
    :param url:
        Django url to be reversed.

    :param css:
        Css class to be returned for highlighting. Return active if none set.

    """
    request = context['request']
    if request.get_full_path in (reverse(url) for url in urls.split()):
        return css if css else 'active'
    return ''

@register.simple_tag(takes_context=True)
def active_path(context, pattern, css=None):
    """
    Highlight menu item based on path.
    
    Returns a css class if ``request.path`` is in given ``pattern``.
    
    :param pattern:
        Regex url pattern.

    :param css:
        Css class to be returned for highlighting. Return active if none set.

    """
    request = context['request']
    #pattern = "^" + pattern + "$"
    if re.search(pattern, request.path):
        return css if css else 'active'
    return ''



# -*- coding: utf-8 -*-
from functools import wraps
from django.http import HttpResponseForbidden
from django.shortcuts import get_object_or_404
from django.utils.decorators import available_attrs

def owner_required(Model=None):
    """
    Usage:
    
    @owner_required(Entry)
    def manage_entry(request, pk=None, object=None):
    
    @owner_required()
    def entry_delete(*args, **kwargs):
        kwargs["post_delete_redirect"] = reverse('manage_blogs')
        return delete_object(*args, **kwargs)
    """
    def decorator(view_func):
        def _wrapped_view(request, *args, **kwargs):
            user = request.user
            grant = False
            model = Model
            mod_edit = False
            if 'pk' in kwargs:
                pk = int(kwargs['pk'])
                if model:
                    mod_edit = True
                elif 'model' in kwargs:
                    model = kwargs['model']
                object = get_object_or_404(model, pk=pk)                
                if user.is_superuser:
                    grant = True
                else:
                    if user.__class__ == model:
                        grant = pk == user.id
                    else:
                        names = [rel.get_accessor_name() 
                                    for rel 
                                    in user._meta.get_all_related_objects() 
                                    if rel.model == model]
                        if names:
                            grant = pk in [o.id 
                                            for o 
                                            in getattr(user, names[0]).all()]
                if not grant:
                    return HttpResponseForbidden('Forbidden')
                if mod_edit:
                    kwargs['object'] = object
            return view_func(request, *args, **kwargs)
        return wraps(view_func)(_wrapped_view)
    return decorator



# -*- coding: utf-8 -*-
import datetime

from django.db import models
from django.conf import settings
from django.core.mail import send_mail
from django.core.files.storage import default_storage
from django.contrib.sites.models import Site
from django.contrib.auth.models import AbstractUser
from django.utils.translation import ugettext_lazy as _
from django.template.loader import render_to_string

from timezone_field import TimeZoneField

from manifest.accounts import defaults
from manifest.accounts.managers import UserManager
from manifest.accounts.utils import (get_gravatar, generate_sha1,   
                                        get_protocol, get_datetime_now)

class AccountActivationMixin(models.Model):
    """
    A mixin that adds the field and methods necessary to support
    account activation.
    """
    activation_key = models.CharField(_(u'Activation key'), 
                        max_length=40, blank=True)

    class Meta:
        abstract = True        

    def activation_key_expired(self):
        """
        Checks if activation key is expired.

        Returns ``True`` when the ``activation_key`` of the user is expired 
        and ``False`` if the key is still valid.

        The key is expired when it's set to the value defined in
        ``ACCOUNTS_ACTIVATED`` or ``activation_key_created`` is beyond the
        amount of days defined in ``ACCOUNTS_ACTIVATION_DAYS``.

        """
        expiration_days = datetime.timedelta(
                            days=defaults.ACCOUNTS_ACTIVATION_DAYS)
        expiration_date = self.date_joined + expiration_days
        if self.activation_key == defaults.ACCOUNTS_ACTIVATED:
            return True
        if get_datetime_now() >= expiration_date:
            return True
        return False

    def send_activation_email(self):
        """
        Sends a activation email to the user.

        This email is send when the user wants to activate their 
        newly created user.

        """
        context= {'user': self,
                  'protocol': get_protocol(),
                  'activation_days': defaults.ACCOUNTS_ACTIVATION_DAYS,
                  'activation_key': self.activation_key,
                  'site': Site.objects.get_current()}

        subject = ''.join(render_to_string(
                    'accounts/emails/activation_email_subject.txt',
                    context).splitlines())

        message = render_to_string(
                    'accounts/emails/activation_email_message.txt',
                    context)
        send_mail(subject,
                  message,
                  settings.DEFAULT_FROM_EMAIL,
                  [self.email,])

class EmailConfirmationMixin(models.Model):
    """
    A mixin that adds the field and methods necessary to support
    e-mail address confirmation.
    """
    email_unconfirmed = models.EmailField(_(u'Unconfirmed email address'), 
                            blank=True,
                            help_text=_(u'Temporary email address when the '
                                'user requests an email change.'))

    email_confirmation_key = models.CharField(_(u'Unconfirmed email '
                                                    'verification key'), 
                                max_length=40, blank=True)

    email_confirmation_key_created = models.DateTimeField(_(u'Creation date '
                                                'of email confirmation key'),
                                        blank=True, null=True)

    class Meta:
        abstract = True        

    def change_email(self, email):
        """
        Changes the email address for a user.

        A user needs to verify this new email address before it becomes
        active. By storing the new email address in a temporary field 
        -- ``temporary_email`` -- we are able to set this email address 
        after the user has verified it by clicking on the verification URI 
        in the email. This email gets send out by ``send_verification_email``.

        :param email:
            The new email address that the user wants to use.

        """
        self.email_unconfirmed = email

        salt, hash = generate_sha1(self.username)
        self.email_confirmation_key = hash
        self.email_confirmation_key_created = get_datetime_now()
        self.save()

        # Send email for activation
        self.send_confirmation_email()
        
        return self

    def send_confirmation_email(self):
        """
        Sends an email to confirm the new email address.

        This method sends out two emails. One to the new email address that
        contains the ``email_confirmation_key`` which is used to verify this
        this email address with :func:`User.objects.confirm_email`.

        The other email is to the old email address to let the user know that
        a request is made to change this email address.

        """
        context= {'user': self,
                  'new_email': self.email_unconfirmed,
                  'protocol': get_protocol(),
                  'confirmation_key': self.email_confirmation_key,
                  'site': Site.objects.get_current()}


        # Email to the old address
        subject_old = ''.join(render_to_string(
                        'accounts/emails/confirmation_email_subject_old.txt',
                        context).splitlines())
        
        message_old = render_to_string(
                        'accounts/emails/confirmation_email_message_old.txt',
                        context)

        send_mail(subject_old,
                  message_old,
                  settings.DEFAULT_FROM_EMAIL,
                  [self.email])

        # Email to the new address
        subject_new = ''.join(render_to_string(
                        'accounts/emails/confirmation_email_subject_new.txt',
                        context).splitlines())

        message_new = render_to_string(
                        'accounts/emails/confirmation_email_message_new.txt',
                        context)

        send_mail(subject_new,
                  message_new,
                  settings.DEFAULT_FROM_EMAIL,
                  [self.email_unconfirmed,])

                  
def upload_to_picture(instance, filename):
    """
    Uploads a picture for a user to the ``ACCOUNTS_PICTURE_PATH`` and 
    saving it under unique hash for the image. This is for privacy 
    reasons so others can't just browse through the picture directory.

    """
    extension = filename.split('.')[-1].lower()
    salt, hash = generate_sha1(instance.id)
    return '%(path)s/%(hash)s.%(extension)s' % {
                'path': getattr(defaults, 
                            'ACCOUNTS_PICTURE_PATH','%s/%s' % (
                                str(instance._meta.app_label), 
                                str(instance._meta.model_name))),
                'hash': hash[:10],
                'extension': extension}
    

class UserProfileMixin(models.Model):
    """ 
    Base model needed for extra profile functionality 
    
    """

    GENDER_CHOICES = (
        ('F', _(u'Female')),
        ('M', _(u'Male')),
    )

    gender = models.CharField(_(u'Gender'), choices=GENDER_CHOICES, 
                max_length=1, blank=True, null=True)    
    birth_date = models.DateField(_(u'Birth date'), blank=True, null=True)
    picture = models.ImageField(_(u'Picture'), blank=True, 
                upload_to=upload_to_picture)


    class Meta:
        abstract = True

    def get_absolute_url(self):
        return ('accounts_profile_detail', None, { 
                    'username': self.username })

    def save(self, force_insert=False, force_update=False, *args, **kwargs):
        try:
            old_obj = self.__class__.objects.get(pk=self.pk)
            if old_obj.picture.path != self.picture.path:
                path = old_obj.picture.path
                default_storage.delete(path)
        except:
            pass
        super(UserProfileMixin, self).save(force_insert, force_update, 
                                        *args, **kwargs)

    @property
    def avatar(self):
        return self.get_picture_url()

    @property
    def age(self):
        TODAY = datetime.date.today()
        if self.birth_date:
            return u"%s" % relativedelta.relativedelta(TODAY, 
                                self.birth_date).years
        else:
            return None
    
    def get_picture_url(self):
        """
        Returns the image containing the picture for the user.

        The picture can be a uploaded image or a Gravatar.

        Gravatar functionality will only be used when
        ``ACCOUNTS_GRAVATAR_PICTURE`` is set to ``True``.

        :return:
            ``None`` when Gravatar is not used and no default image is 
            supplied by ``ACCOUNTS_GRAVATAR_DEFAULT``.

        """
        # First check for a picture and if any return that.
        if self.picture:
            return self.picture.url

        # Use Gravatar if the user wants to.
        if defaults.ACCOUNTS_GRAVATAR_PICTURE:
            return get_gravatar(self.email,
                                defaults.ACCOUNTS_GRAVATAR_SIZE,
                                defaults.ACCOUNTS_GRAVATAR_DEFAULT)

        # Gravatar not used, check for a default image.
        else:
            if defaults.ACCOUNTS_GRAVATAR_DEFAULT not in ['404', 'mm',
                                                                'identicon',
                                                                'monsterid',
                                                                'wavatar']:
                return defaults.ACCOUNTS_GRAVATAR_DEFAULT
            else: return None

    def get_full_name_or_username(self):
        """
        Returns the full name of the user, or if none is supplied will return
        the username.

        Also looks at ``ACCOUNTS_WITHOUT_USERNAMES`` settings to define if it
        should return the username or email address when the full name is not
        supplied.

        :return:
            ``String`` containing the full name of the user. If no name is
            supplied it will return the username or email address depending on
            the ``ACCOUNTS_WITHOUT_USERNAMES`` setting.

        """
        if self.first_name or self.last_name:
            # We will return this as translated string. Maybe there are some
            # countries that first display the last name.
            name = _(u"%(first_name)s %(last_name)s") % \
                {'first_name': self.first_name,
                 'last_name': self.last_name}
        else:
            # Fallback to the username if usernames are used
            if not defaults.ACCOUNTS_WITHOUT_USERNAMES:
                name = "%(username)s" % {'username': self.username}
            else:
                name = "%(email)s" % {'email': self.email}
        return name.strip()


class UserLocaleMixin(models.Model):
    """
    A mixin that adds the field and methods necessary to support
    i18n & L10n for UI.
    """
    timezone = TimeZoneField(_(u"Timezone"))

    locale = models.CharField(_(u"Locale"), max_length = 10,
                                    choices = settings.LANGUAGES,
                                    default = settings.LANGUAGE_CODE)
    class Meta:
        abstract = True

    
class BaseUser(AccountActivationMixin, EmailConfirmationMixin, 
                    UserProfileMixin, UserLocaleMixin):
    """
    Custom user model base which stores all the necessary information 
    to have a fully functional user implementation on your Django website.
    """

    class Meta:
        abstract = True
        

class User(AbstractUser, BaseUser):
    
    objects = UserManager()
    
    class Meta:
        swappable = 'AUTH_USER_MODEL'


def create_user(sender, user, *args, **kwargs):
    user = User.objects.activate_user(user.username, 
                                            user.activation_key)
    
if 'social_auth' in settings.INSTALLED_APPS:
    from social_auth.signals import socialauth_registered

    socialauth_registered.connect(create_user, sender=None)





from django.urls import reverse

#from manifest.accounts import defaults
from manifest.accounts.tests.base import AccountsTestCase

class DecoratorTests(AccountsTestCase):
    """ Test the decorators """

    def test_secure_required(self):
        """
        Test that the ``secure_required`` decorator does a permanent redirect
        to a secured page.

        """
        # defaults.ACCOUNTS_USE_HTTPS = True
        with self.settings(ACCOUNTS_USE_HTTPS=True):
            response = self.client.get(reverse('accounts_login'))
            # Test for the permanent redirect
            self.assertEqual(response.status_code, 301)
            # Test if the redirected url contains 'https'. Couldn't use
            # ``assertRedirects`` here because the redirected to page is
            # non-existant.
            self.assertTrue('https' in response.get('Location'))
            # Set back to the old settings
            # defaults.ACCOUNTS_USE_HTTPS = False